==> jalview_2_04.tar.gz: jalview/src/jalview/appletgui/OverviewPanel.java <==
/*
 * Jalview - A Sequence Alignment Editor and Viewer
 * Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

package jalview.appletgui;

import java.awt.*;
import java.awt.event.*;

public class OverviewPanel
    extends Panel implements Runnable
{
  Image miniMe;
  AlignViewport av;
  AlignmentPanel ap;
  float scalew = 1f;
  float scaleh = 1f;

  public int width, sequencesHeight;
  int graphHeight = 30;
  int boxX = -1, boxY = -1, boxWidth = -1, boxHeight = -1;

  boolean resizing = false;

  Frame nullFrame;

  public OverviewPanel(AlignmentPanel ap)
  {
    this.av = ap.av;
    this.ap = ap;
    setLayout(null);
    nullFrame = new Frame();
    nullFrame.addNotify();

    // scale the initial size of overviewpanel to shape of alignment
    float initialScale = (float) av.alignment.getWidth() /
        (float) av.alignment.getHeight();
    if (av.alignment.getWidth() > av.alignment.getHeight())
    {
      // wider
      width = 400;
      sequencesHeight = (int) (400f / initialScale);
    }
    else
    {
      // taller
      width = (int) (400f * initialScale);
      sequencesHeight = 300;
      if (width < 120)
      {
        width = 120;
      }
    }

    setSize(new Dimension(width, sequencesHeight + graphHeight));
    addComponentListener(new ComponentAdapter()
    {

      public void componentResized(ComponentEvent evt)
      {
        if (getSize().width != width ||
            getSize().height != sequencesHeight + graphHeight)
        {
          updateOverviewImage();
        }
      }
    });

    addMouseMotionListener(new MouseMotionAdapter()
    {
      public void mouseDragged(MouseEvent evt)
      {
        doMouseDragged(evt);
      }
    });

    addMouseListener(new MouseAdapter()
    {
      public void mousePressed(MouseEvent evt)
      {
        doMousePressed(evt);
      }

      public void mouseReleased(MouseEvent evt)
      {
        doMouseReleased(evt);
      }
    });

    updateOverviewImage();

  }

  public void doMousePressed(MouseEvent evt)
  {
    boxX = evt.getX();
    boxY = evt.getY();

    checkValid();
    repaint();
  }

  public void doMouseReleased(MouseEvent evt)
  {
    boxX = evt.getX();
    boxY = evt.getY();
    checkValid();
    if (!resizing)
    {
      ap.setScrollValues( (int) (boxX / scalew / av.getCharWidth()),
                         (int) (boxY / scaleh / av.getCharHeight()));
    }
  }

  public void doMouseDragged(MouseEvent evt)
  {
    boxX = evt.getX();
    boxY = evt.getY();
    checkValid();
    if (!resizing)
    {
      ap.setScrollValues( (int) (boxX / scalew / av.getCharWidth()),
                         (int) (boxY / scaleh / av.getCharHeight()));
    }
    repaint();
    ap.repaint();
  }

  void checkValid()
  {
    if (boxY < 0)
    {
      boxY = 0;
    }

    if (boxY > sequencesHeight - boxHeight)
    {
      boxY = sequencesHeight - boxHeight + 1;
    }

    if (boxX < 0)
    {
      boxX = 0;
    }

    if (boxX > width - boxWidth)
    {
      boxX = width - boxWidth;
    }
  }

  public void updateOverviewImage()
  {
    if (resizing)
    {
      return;
    }

    resizing = true;
    Thread thread = new Thread(this);
    thread.start();
    repaint();
  }

  public void run()
  {
    miniMe = null;
    int alwidth = av.alignment.getWidth();
    int alheight = av.alignment.getHeight();

    if (getSize().width > 0 && getSize().height > 0)
    {
      width = getSize().width;
      sequencesHeight = getSize().height - graphHeight;
    }

    setSize(new Dimension(width, sequencesHeight + graphHeight));

    int fullsizeWidth = alwidth * av.getCharWidth();
    int fullsizeHeight = alheight * av.getCharHeight();

    scalew = (float) width / (float) fullsizeWidth;
    scaleh = (float) sequencesHeight / (float) fullsizeHeight;

    miniMe = nullFrame.createImage(width, sequencesHeight + graphHeight);

    Graphics mg = miniMe.getGraphics();
    Image consensus = nullFrame.createImage(fullsizeWidth, 60);
    Graphics g = consensus.getGraphics();
    ap.annotationPanel.drawGraph(g, av.conservation, fullsizeWidth, 60);
    mg.drawImage(consensus, 0, sequencesHeight, width,
                 sequencesHeight + graphHeight, 0, 0, fullsizeWidth, 60, this);

    boolean oldRenderGaps = av.renderGaps;
    try
    {
      // We'll have to draw the full size alignment in chunks, as an image of the
      // whole alignment requires too much memory

      // Max size depends on the font size, the following is a
      // guess at a size which works
      int maxSize = 2000 / av.getFont().getSize();
      Image block;
      int blockx = 0, blocky = 0, blockw = 0, blockh = 0, eRes = 0, eSeq = 0;

      av.setRenderGaps(false);
      for (int sRes = 0, chunkx = 0; sRes < alwidth; sRes += maxSize, chunkx++)
      {
        eSeq = 0;
        eRes += maxSize;
        if (eRes > alwidth)
        {
          eRes = alwidth;
        }

        for (int sSeq = 0, chunky = 0; sSeq < alheight; sSeq += maxSize, chunky++)
        {
          eSeq += maxSize;
          if (eSeq > alheight)
          {
            eSeq = alheight;
          }

          blocky = 0;
          blockx = (int) ( (float) sRes / (float) alwidth * width);

          block = nullFrame.createImage( (eRes - sRes) * av.charWidth,
                                        (eSeq - sSeq) * av.charHeight);
          g = block.getGraphics();

          ap.seqPanel.seqCanvas.drawPanel(g, sRes, eRes, sSeq, eSeq, sRes, sSeq,
                                          0);

          blockh = (int) ( (float) (eSeq - sSeq) / (float) alheight *
                          sequencesHeight) + 1;
          blockw = (int) ( (float) (eRes - sRes) / (float) alwidth * width) + 1;

          blocky += (int) ( (float) sSeq / (float) alheight * sequencesHeight);

          mg.drawImage(block, blockx,
                       blocky,
                       blockx + blockw,
                       blocky + blockh,

                       0, 0, block.getWidth(null), block.getHeight(null), this);

          block = null;
        }

      }

    }
    catch (OutOfMemoryError error)
    {
      System.err.println(
          "Out of memory when trying to calculate the overview window image!");
    }

    av.setRenderGaps(oldRenderGaps);
    resizing = false;

    setBoxPosition();
  }

  public void setBoxPosition()
  {
    boxX = (int) (av.getStartRes() * av.getCharWidth() * scalew);
    boxY = (int) (av.getStartSeq() * av.getCharHeight() * scaleh);
    boxWidth = (int) ( (av.getEndRes() - av.getStartRes() + 1) *
                      av.getCharWidth() * scalew);
    boxHeight = (int) (av.getEndSeq() * av.getCharHeight() * scaleh) - boxY;
    repaint();
  }

  public void update(Graphics g)
  {
    paint(g);
  }

  public void paint(Graphics g)
  {
    if (resizing)
    {
      g.setColor(Color.white);
      g.fillRect(0, 0, getSize().width, getSize().height);
      g.setColor(Color.black);

      g.setFont(new Font("Verdana", Font.BOLD, 15));
      g.drawString("Recalculating", 5, sequencesHeight / 2);
      g.drawString("Overview.....", 5, sequencesHeight / 2 + 20);
    }
    else
    {
      if (miniMe != null)
      {
        g.drawImage(miniMe, 0, 0, this);
      }

      g.setColor(Color.red);
      g.drawRect(boxX, boxY, boxWidth, boxHeight);
      g.drawRect(boxX + 1, boxY + 1, boxWidth - 2, boxHeight - 2);
    }
  }

}

==> jalview_2_05b.tar.gz: jalview/src/jalview/appletgui/OverviewPanel.java <==
/*
 * Jalview - A Sequence Alignment Editor and Viewer
 * Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

package jalview.appletgui;

import java.awt.*;
import java.awt.event.*;

public class OverviewPanel
    extends Panel implements Runnable
{
  Image miniMe;
  AlignViewport av;
  AlignmentPanel ap;
  float scalew = 1f;
  float scaleh = 1f;

  public int width, sequencesHeight;
  int graphHeight = 20;
  int boxX = -1, boxY = -1, boxWidth = -1, boxHeight = -1;

  boolean resizing = false;

  // Can set different properties in this seqCanvas than
  // main visible SeqCanvas
  SeqCanvas overviewSeq;


  Frame nullFrame;

  public OverviewPanel(AlignmentPanel ap)
  {
    this.av = ap.av;
    this.ap = ap;
    setLayout(null);
    nullFrame = new Frame();
    nullFrame.addNotify();

    overviewSeq = new SeqCanvas(av);
    overviewSeq.isOverview = true;
    overviewSeq.sr.renderGaps = false;


    // scale the initial size of overviewpanel to shape of alignment
    float initialScale = (float) av.alignment.getWidth() /
        (float) av.alignment.getHeight();

    if(av.vconsensus==null)
          graphHeight = 0;

    if (av.alignment.getWidth() > av.alignment.getHeight())
    {
      // wider
      width = 400;
      sequencesHeight = (int) (400f / initialScale);
      if(sequencesHeight<40)
              sequencesHeight = 40;
    }
    else
    {
      // taller
      width = (int) (400f * initialScale);
      sequencesHeight = 300;
      if (width < 120)
      {
        width = 120;
      }
    }

    setSize(new Dimension(width, sequencesHeight + graphHeight));
    addComponentListener(new ComponentAdapter()
    {

      public void componentResized(ComponentEvent evt)
      {
        if (getSize().width != width ||
            getSize().height != sequencesHeight + graphHeight)
        {
          updateOverviewImage();
        }
      }
    });

    addMouseMotionListener(new MouseMotionAdapter()
    {
      public void mouseDragged(MouseEvent evt)
      {
        doMouseDragged(evt);
      }
    });

    addMouseListener(new MouseAdapter()
    {
      public void mousePressed(MouseEvent evt)
      {
        doMousePressed(evt);
      }

      public void mouseReleased(MouseEvent evt)
      {
        doMouseReleased(evt);
      }
    });

    updateOverviewImage();

  }

  public void doMousePressed(MouseEvent evt)
  {
    boxX = evt.getX();
    boxY = evt.getY();

    checkValid();
    repaint();
  }

  public void doMouseReleased(MouseEvent evt)
  {
    boxX = evt.getX();
    boxY = evt.getY();
    checkValid();
    ap.setScrollValues( (int) (boxX / scalew / av.getCharWidth()),
                         (int) (boxY / scaleh / av.getCharHeight()));
  }

  public void doMouseDragged(MouseEvent evt)
  {
    boxX = evt.getX();
    boxY = evt.getY();
    checkValid();
    ap.setScrollValues( (int) (boxX / scalew / av.getCharWidth()),
                         (int) (boxY / scaleh / av.getCharHeight()));

    repaint();
    ap.repaint();
  }

  void checkValid()
  {
    if (boxY < 0)
    {
      boxY = 0;
    }

    if (boxY > sequencesHeight - boxHeight)
    {
      boxY = sequencesHeight - boxHeight + 1;
    }

    if (boxX < 0)
    {
      boxX = 0;
    }

    if (boxX > width - boxWidth)
    {
      boxX = width - boxWidth;
    }
  }

  /**
   * DOCUMENT ME!
   */
  public void updateOverviewImage()
  {
    if (resizing)
    {
        resizeAgain = true;
        return;
    }

    resizing = true;

    if ( (getSize().width > 0) && (getSize().height > 0))
    {
      width = getSize().width;
      sequencesHeight = getSize().height - graphHeight;
    }
    setSize(new Dimension(width, sequencesHeight + graphHeight));
    setBoxPosition();

    Thread thread = new Thread(this);
    thread.start();
        repaint();
  }

  // This is set true if the user resizes whilst
  // the overview is being calculated
    boolean resizeAgain = false;

  public void run()
  {
    miniMe = null;
    int alwidth = av.alignment.getWidth();
    int alheight = av.alignment.getHeight();

    if (getSize().width > 0 && getSize().height > 0)
    {
      width = getSize().width;
      sequencesHeight = getSize().height - graphHeight;
    }

    setSize(new Dimension(width, sequencesHeight + graphHeight));

    int fullsizeWidth = alwidth * av.getCharWidth();
    int fullsizeHeight = alheight * av.getCharHeight();

    scalew = (float) width / (float) fullsizeWidth;
    scaleh = (float) sequencesHeight / (float) fullsizeHeight;

    miniMe = nullFrame.createImage(width, sequencesHeight + graphHeight);

    Graphics mg = miniMe.getGraphics();
    float sampleCol = (float) alwidth / (float) width;
    float sampleRow = (float) alheight / (float) sequencesHeight;

    for (int col = 0; col < width; col++)
    {
      for (int row = 0; row < sequencesHeight; row++)
      {
        overviewSeq.drawPanel(mg,
                                        (int) (col * sampleCol),
                                        (int) (col * sampleCol),
                                        (int) (row * sampleRow),
                                        (int) (row * sampleRow) + 1,
                                        (int) (col * sampleCol),
                                        (int) (row * sampleRow), 0);
        mg.translate(0, 1);

        if (av.conservation != null)
          ap.annotationPanel.drawGraph(mg, av.conservation,
                                       (int) (sampleCol) + 1,
                                       graphHeight,
                                       (int) (col * sampleCol),
                                       (int) (col * sampleCol) + 1);

      }
      mg.translate(0, -sequencesHeight);
      mg.translate(1, 0);

        }
    System.gc();

    resizing = false;

    setBoxPosition();

    if(resizeAgain)
    {
      resizeAgain = false;
      updateOverviewImage();
    }
  }

  public void setBoxPosition()
  {
    int fullsizeWidth = av.alignment.getWidth() * av.getCharWidth();
    int fullsizeHeight = av.alignment.getHeight() * av.getCharHeight();

    scalew = (float) width / (float) fullsizeWidth;
    scaleh = (float) sequencesHeight / (float) fullsizeHeight;

    boxX = (int) (av.getStartRes() * av.getCharWidth() * scalew);
    boxY = (int) (av.getStartSeq() * av.getCharHeight() * scaleh);
    boxWidth = (int) ( (av.getEndRes() - av.getStartRes() + 1) *
                      av.getCharWidth() * scalew);
    boxHeight = (int) (av.getEndSeq() * av.getCharHeight() * scaleh) - boxY;
    repaint();
  }

  public void update(Graphics g)
  {
    paint(g);
  }

  public void paint(Graphics g)
  {
    if (miniMe != null)
    {
      g.drawImage(miniMe, 0, 0, this);
    }
    else
    {
      g.setColor(Color.white);
      g.fillRect(0, 0, getSize().width, getSize().height);
      g.setColor(Color.black);
      g.setFont(new Font("Verdana", Font.BOLD, 15));
      g.drawString("Recalculating", 5, sequencesHeight / 2);
      g.drawString("Overview.....", 5, (sequencesHeight / 2) + 20);
    }

    g.setColor(Color.red);
    g.drawRect(boxX, boxY, boxWidth, boxHeight);
    g.drawRect(boxX + 1, boxY + 1, boxWidth - 2, boxHeight - 2);
  }

}
